@@ -46,7 +46,11 @@ class Feed(object):
             activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
             activity_id = feed.add_activity(activity_data)
         '''
+        if activity_data.get('to') and not isinstance(activity_data.get('to'), (list, tuple, set)):
+            raise TypeError('please provide the activity\'s to field as a list not a string')
+
         if activity_data.get('to'):
+            activity_data = activity_data.copy()
             activity_data['to'] = self.add_to_signature(activity_data['to'])
 
         token = self.create_scope_token('feed', 'write')
@@ -68,12 +72,15 @@ class Feed(object):
             ]
             result = feed.add_activities(activity_data)
         '''
+        activities = []
         for activity_data in activity_list:
+            activity_data = activity_data.copy()
+            activities.append(activity_data)
             if activity_data.get('to'):
                 activity_data['to'] = self.add_to_signature(
                     activity_data['to'])
         token = self.create_scope_token('feed', 'write')
-        data = dict(activities=activity_list)
+        data = dict(activities=activities)
         result = self.client.post(
             self.feed_url, data=data, signature=token)
         return result
