@@ -3,19 +3,24 @@ import stream
 import time
 from stream.exceptions import ApiKeyException, InputException
 import random
+import jwt
 from unittest.case import TestCase
 
 import os
 import datetime
+import copy
 from stream import serializer
 from requests.exceptions import ConnectionError, MissingSchema
-from urlparse import urlparse
 
+try:
+    from urlparse import urlparse, parse_qs
+except ImportError:
+    from urllib.parse import urlparse, parse_qs
 
 def connect_debug():
     return stream.connect(
-        'ahj2ndz7gsan',
-        'gthc2t9gh7pzq52f6cky8w4r4up9dr6rju9w3fjgmkv6cdvvav2ufe5fv7e2r9qy',
+        'gav9ygr75by3',
+        '5ws2hnua79n9qga6e2dy572qdfapwgdc83853mjm3mjp66czyb2xkahbdhs98an8',
         location='us-east',
         timeout=10
     )
@@ -37,7 +42,6 @@ aggregated3 = getfeed('aggregated', '3')
 topic1 = getfeed('topic', '1')
 flat3 = getfeed('flat', '3')
 
-
 class ClientTest(TestCase):
 
     def setUp(self):
@@ -50,6 +54,51 @@ class ClientTest(TestCase):
         self.topic1 = topic1
         self.flat3 = flat3
 
+    def test_update_activities_create(self):
+        activities = [{
+            'actor': 'user:1',
+            'verb': 'do',
+            'object': 'object:1',
+            'foreign_id': 'object:1',
+            'time': datetime.datetime.utcnow().isoformat()
+        }]
+
+        self.c.update_activities(activities)
+
+    def test_update_activities_illegal_argument(self):
+        activities = dict()
+
+        def invalid_activities():
+            self.c.update_activities(activities)
+        self.assertRaises(TypeError, invalid_activities)
+
+    def test_update_activities_update(self):
+        activities = []
+        for i in range(0, 10):
+            activities.append({
+                'actor': 'user:1',
+                'verb': 'do',
+                'object': 'object:%s' % i,
+                'foreign_id': 'object:%s' % i,
+                'time': datetime.datetime.utcnow().isoformat()
+            })
+        activities_created = user1.add_activities(activities)['activities']
+        activities = copy.deepcopy(activities_created)
+
+        time.sleep(3)
+        for activity in activities:
+            activity.pop('id')
+            activity['popularity'] = 100
+
+        self.c.update_activities(activities)
+
+        activities_updated = user1.get(limit=len(activities))['results']
+        activities_updated.reverse()
+
+        for i, activity in enumerate(activities_updated):
+            self.assertEqual(activities_created[i].get('id'), activity.get('id'))
+            self.assertEquals(activity['popularity'], 100)
+
     def test_heroku(self):
         url = 'https://thierry:pass@getstream.io/?app_id=1'
         os.environ['STREAM_URL'] = url
@@ -127,6 +176,24 @@ class ClientTest(TestCase):
         activities = feed.get(limit=1)['results']
         self.assertEqual(activities[0]['id'], activity_id)
 
+    def test_add_activity_to_inplace_change(self):
+        feed = getfeed('user', 'py1')
+        team_feed = getfeed('user', 'teamy')
+        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
+        activity_data['to'] = [team_feed.id]
+        feed.add_activity(activity_data)
+        time.sleep(2)
+        self.assertEqual(activity_data['to'], [team_feed.id])
+
+    def test_add_activities_to_inplace_change(self):
+        feed = getfeed('user', 'py1')
+        team_feed = getfeed('user', 'teamy')
+        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
+        activity_data['to'] = [team_feed.id]
+        feed.add_activities([activity_data])
+        time.sleep(2)
+        self.assertEqual(activity_data['to'], [team_feed.id])
+
     def test_add_activity_to(self):
         # test for sending an activities to the team feed using to
         feeds = ['user', 'teamy', 'team_follower']
@@ -155,6 +222,17 @@ class ClientTest(TestCase):
         # and the flat feed
         activities = team_follower_feed.get(limit=1)['results']
         self.assertFirstActivityIDNotEqual(activities, activity_id)
+
+
+    def test_add_activity_to_type_error(self):
+        user_feed = getfeed('user', '1')
+        activity_data = {
+            'actor': 1, 'verb': 'tweet', 'object': 1,
+            'to': 'string'
+        }
+
+        with self.assertRaises(TypeError):
+            user_feed.add_activity(activity_data)
         
     def assertFirstActivityIDEqual(self, activities, correct_activity_id):
         activity_id = None
@@ -248,18 +326,18 @@ class ClientTest(TestCase):
             self.assertNotEqual(activity_id_found, activity_id)
             time.sleep(1)
 
-    def test_follow_private(self):
-        feed = getfeed('secret', 'py1')
-        agg_feed = getfeed('aggregated', 'test_follow_private')
-        actor_id = random.randint(10, 100000)
-        activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
-        activity_id = feed.add_activity(activity_data)['id']
-        agg_feed.follow(feed.slug, feed.user_id)
-        time.sleep(10)
-        activities = agg_feed.get(limit=3)['results']
-        activity = self._get_first_aggregated_activity(activities)
-        activity_id_found = activity['id'] if activity is not None else None
-        self.assertEqual(activity_id_found, activity_id)
+    # def test_follow_private(self):
+    #     feed = getfeed('secret', 'py1')
+    #     agg_feed = getfeed('aggregated', 'test_follow_private')
+    #     actor_id = random.randint(10, 100000)
+    #     activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
+    #     activity_id = feed.add_activity(activity_data)['id']
+    #     agg_feed.follow(feed.slug, feed.user_id)
+    #     time.sleep(10)
+    #     activities = agg_feed.get(limit=3)['results']
+    #     activity = self._get_first_aggregated_activity(activities)
+    #     activity_id_found = activity['id'] if activity is not None else None
+    #     self.assertEqual(activity_id_found, activity_id)
 
     def test_flat_follow(self):
         feed = getfeed('user', 'test_flat_follow')
@@ -626,7 +704,6 @@ class ClientTest(TestCase):
             'auth_type=jwt',
             'url=http%3A%2F%2Fgoogle.com%2F%3Fa%3Db%26c%3Dd',
             'api_key=ahj2ndz7gsan',
-            'authorization=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJhY3Rpb24iOiIqIiwidXNlcl9pZCI6InRvbW1hc28iLCJyZXNvdXJjZSI6InJlZGlyZWN0X2FuZF90cmFjayJ9.pQ2cBmC7l0WGP9LP7RrvLEbtrw8YWcFtjOqSfoSr2s0',
             'events=%5B%7B%22foreign_ids%22%3A+%5B%22tweet%3A1%22%2C+%22tweet%3A2%22%2C+%22tweet%3A3%22%2C+%22tweet%3A4%22%2C+%22tweet%3A5%22%5D%2C+%22feed_id%22%3A+%22user%3Aglobal%22%2C+%22user_id%22%3A+%22tommaso%22%2C+%22location%22%3A+%22email%22%7D%2C+%7B%22user_id%22%3A+%22tommaso%22%2C+%22label%22%3A+%22click%22%2C+%22feed_id%22%3A+%22user%3Aglobal%22%2C+%22location%22%3A+%22email%22%2C+%22position%22%3A+3%2C+%22foreign_id%22%3A+%22tweet%3A1%22%7D%5D',
         ]
         engagement = {'foreign_id': 'tweet:1', 'label': 'click', 'position': 3, 'user_id': 'tommaso', 'location': 'email', 'feed_id': 'user:global'}
@@ -637,6 +714,17 @@ class ClientTest(TestCase):
         user_id = 'tommaso'
         redirect_url = self.c.create_redirect_url(target_url, user_id, events)
 
+        parsed_url = urlparse(redirect_url)
+        qs = parse_qs(parsed_url.query)
+
+        decoded = jwt.decode(qs['authorization'][0], 'gthc2t9gh7pzq52f6cky8w4r4up9dr6rju9w3fjgmkv6cdvvav2ufe5fv7e2r9qy')
+
+        self.assertEqual(decoded, {
+            'resource': 'redirect_and_track',
+            'action': '*',
+            'user_id': 'tommaso'
+        })
+
         for part in expected_parts:
             if part not in redirect_url:
                 raise ValueError('didnt find %s in url \n %s' % (part, redirect_url))
